package ink.wgink.module.activiti.service.activiti.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.module.activiti.pojo.dtos.ActivitiFlowNodeDTO;
import ink.wgink.module.activiti.pojo.dtos.ActivitiProcdefDTO;
import ink.wgink.module.activiti.service.activiti.IActivitiProcdefService;
import ink.wgink.pojo.result.SuccessResultList;
import org.activiti.bpmn.model.Process;
import org.activiti.bpmn.model.*;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: ActivitiProcdefServiceImpl
 * @Description: 流程定义
 * @Author: wanggeng
 * @Date: 2021/12/7 10:11 PM
 * @Version: 1.0
 */
@Service
public class ActivitiProcdefServiceImpl extends DefaultBaseService implements IActivitiProcdefService {

    @Autowired
    private RepositoryService repositoryService;

    @Override
    public SuccessResultList<List<ActivitiProcdefDTO>> listPage(int page, int rows) {
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.listPage(page - 1, rows);

        List<ActivitiProcdefDTO> activitiProcdefDTOs = new ArrayList<>();
        processDefinitions.forEach(processDefinition -> {
            ActivitiProcdefDTO activitiProcdefDTO = new ActivitiProcdefDTO();
            BeanUtils.copyProperties(processDefinition, activitiProcdefDTO);
            activitiProcdefDTOs.add(activitiProcdefDTO);
        });

        return new SuccessResultList<>(activitiProcdefDTOs, page, processDefinitionQuery.count());
    }

    @Override
    public List<ActivitiFlowNodeDTO> listFlowNodeByDeploymentId(String deploymentId) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().deploymentId(deploymentId).singleResult();
        if (processDefinition == null) {
            throw new SearchException("流程定义不存在，请检查流程部署情况");
        }
        BpmnModel bpmnModel = repositoryService.getBpmnModel(processDefinition.getId());
        return listFlowNodeByModel(bpmnModel);
    }

    @Override
    public List<ActivitiFlowNodeDTO> listFlowNodeByModel(BpmnModel bpmnModel) {
        List<Process> processes = bpmnModel.getProcesses();
        if (processes.size() > 1) {
            throw new SystemException("流程不能有多条");
        }
        List<FlowNode> flowNodes = processes.get(0).findFlowElementsOfType(FlowNode.class);
        List<ActivitiFlowNodeDTO> activitiFlowNodeDTOs = new ArrayList<>();
        for (FlowNode flowNode : flowNodes) {
            ActivitiFlowNodeDTO activitiFlowNodeDTO = new ActivitiFlowNodeDTO();
            activitiFlowNodeDTO.setId(flowNode.getId());
            activitiFlowNodeDTO.setName(flowNode.getName());
            activitiFlowNodeDTO.setSummary(flowNode.getDocumentation());
            if (flowNode instanceof StartEvent) {
                StartEvent startEvent = (StartEvent) flowNode;
                activitiFlowNodeDTO.setFormKey(startEvent.getFormKey());
                activitiFlowNodeDTO.setType(StartEvent.class.getSimpleName());
            } else if (flowNode instanceof EndEvent) {
                activitiFlowNodeDTO.setType(EndEvent.class.getSimpleName());
            } else if (flowNode instanceof UserTask) {
                UserTask userTask = (UserTask) flowNode;
                activitiFlowNodeDTO.setFormKey(userTask.getFormKey());
                activitiFlowNodeDTO.setType(UserTask.class.getSimpleName());
            } else {
                continue;
            }
            activitiFlowNodeDTOs.add(activitiFlowNodeDTO);
        }
        return activitiFlowNodeDTOs;
    }


}
