package ink.wgink.module.activiti.service.activiti.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.activiti.pojo.dtos.ActivitiFlowNodeDTO;
import ink.wgink.module.activiti.pojo.vos.ActivitiVO;
import ink.wgink.module.activiti.pojo.vos.oa.NodeFieldUpdateVO;
import ink.wgink.module.activiti.pojo.vos.oa.NodeFieldVO;
import ink.wgink.module.activiti.service.activiti.IActivitiProcdefService;
import ink.wgink.module.activiti.service.activiti.IActivitiService;
import ink.wgink.module.activiti.service.oa.INodeFieldService;
import ink.wgink.module.form.pojo.pos.design.FormFieldPO;
import ink.wgink.module.form.pojo.pos.design.FormPO;
import ink.wgink.module.form.service.design.IFormFieldService;
import ink.wgink.module.form.service.design.IFormService;
import ink.wgink.pojo.result.SuccessResultList;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.StartEvent;
import org.activiti.editor.constants.ModelDataJsonConstants;
import org.activiti.editor.language.json.converter.BpmnJsonConverter;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.repository.Deployment;
import org.activiti.engine.repository.Model;
import org.activiti.engine.repository.ModelQuery;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ActivitiServiceImpl
 * @Description: activiti
 * @Author: WangGeng
 * @Date: 2021/7/11 16:46
 * @Version: 1.0
 **/
@Service
public class ActivitiServiceImpl extends DefaultBaseService implements IActivitiService {

    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private IActivitiProcdefService activitiProcdefService;
    @Autowired
    private INodeFieldService nodeFieldService;
    @Autowired
    private IFormService formService;
    @Autowired
    private IFormFieldService formFieldService;

    @Override
    public String create(ActivitiVO activitiVO) throws UnsupportedEncodingException {
        long count = repositoryService.createModelQuery().modelName(activitiVO.getModelName()).modelKey(activitiVO.getModelKey()).count();
        if (count >= 1) {
            throw new SearchException("模型已经存在");
        }
        Model model = repositoryService.newModel();
        model.setName(activitiVO.getModelName());
        model.setKey(activitiVO.getModelKey());
        model.setMetaInfo(createModelMetaInfo(activitiVO));
        model.setVersion(1);
        LOG.debug("保存模型");
        repositoryService.saveModel(model);
        addModelEditorSource(model.getId());
        return model.getId();
    }

    @Override
    public void delete(String modelId) {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        if (!StringUtils.isBlank(model.getDeploymentId())) {
            throw new RemoveException("已部署的模型不能删除");
        }
        repositoryService.deleteModel(modelId);
    }

    @Override
    public void saveDeploy(String modelId) throws IOException {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        byte[] modelEditorSource = repositoryService.getModelEditorSource(model.getId());
        if (modelEditorSource == null) {
            throw new SearchException("模型数据不存在");
        }
        LOG.debug("部署流程");
        JsonNode modelNode = new ObjectMapper().readTree(modelEditorSource);
        BpmnModel bpmnModel = new BpmnJsonConverter().convertToBpmnModel(modelNode);
        Deployment deployment = repositoryService.createDeployment()
                .name(model.getName())
                .key(model.getKey())
                .addBpmnModel(model.getKey() + "." + model.getVersion() + ".bpmn20.xml", bpmnModel)
                .deploy();
        model.setDeploymentId(deployment.getId());
        repositoryService.saveModel(model);
        LOG.debug("节点关联表单");
        saveNodeField(deployment.getId(), bpmnModel);
    }

    /**
     * 保存节点与表单字段
     *
     * @param deploymentId
     */
    private void saveNodeField(String deploymentId, BpmnModel bpmnModel) {
        List<ActivitiFlowNodeDTO> activitiFlowNodeDTOs = activitiProcdefService.listFlowNodeByModel(bpmnModel);
        if (activitiFlowNodeDTOs.isEmpty()) {
            return;
        }
        String formKey = getFormIdByStartEvent(activitiFlowNodeDTOs);
        if (StringUtils.isBlank(formKey)) {
            LOG.debug("开始节点未绑定表单");
            return;
        }
        FormPO formPO = formService.getPO(formKey);
        if (formPO == null) {
            throw new SystemException("表单不存在");
        }
        List<FormFieldPO> formFieldPOs = formFieldService.listPO(formPO.getFormId());
        if (formFieldPOs.isEmpty()) {
            throw new SystemException("表单字段为空");
        }
        activitiFlowNodeDTOs.forEach(activitiFlowNodeDTO -> {
            NodeFieldUpdateVO nodeFieldUpdateVO = new NodeFieldUpdateVO();
            nodeFieldUpdateVO.setFlowNodeId(activitiFlowNodeDTO.getId());
            nodeFieldUpdateVO.setFormId(formPO.getFormId());
            nodeFieldUpdateVO.setDeploymentId(deploymentId);
            List<NodeFieldVO> nodeFieldVOs = new ArrayList<>();
            formFieldPOs.forEach(formFieldPO -> {
                NodeFieldVO nodeFieldVO = new NodeFieldVO();
                nodeFieldVO.setFieldName(formFieldPO.getFieldName());
                nodeFieldVO.setIsVisible(1);
                nodeFieldVO.setIsEditable(1);
                nodeFieldVOs.add(nodeFieldVO);
            });
            nodeFieldUpdateVO.setFormFields(nodeFieldVOs);
            nodeFieldService.save(nodeFieldUpdateVO);
        });
    }

    private String getFormIdByStartEvent(List<ActivitiFlowNodeDTO> activitiFlowNodeDTOs) {
        for (ActivitiFlowNodeDTO activitiFlowNodeDTO : activitiFlowNodeDTOs) {
            if (StringUtils.equals(activitiFlowNodeDTO.getType(), StartEvent.class.getSimpleName())) {
                return activitiFlowNodeDTO.getFormKey();
            }
        }
        return null;
    }

    @Override
    public void revokeDeploy(String modelId) {
        Model modelData = repositoryService.getModel(modelId);
        if (modelData == null) {
            throw new SearchException("模型不存在");
        }
        /**
         * 参数不加true:为普通删除，如果当前规则下有正在执行的流程，则抛异常
         * 参数加true:为级联删除,会删除和当前规则相关的所有信息，包括历史
         */
        repositoryService.deleteDeployment(modelData.getDeploymentId());
    }

    @Override
    public List<Model> list() {
        return repositoryService.createModelQuery().list();
    }

    @Override
    public SuccessResultList<List<Model>> listPage(int page, int rows) {
        ModelQuery modelQuery = repositoryService.createModelQuery();
        List<Model> models = modelQuery.listPage(page - 1, rows);
        return new SuccessResultList<>(models, page, modelQuery.count());
    }

    /**
     * 创建模型信息
     *
     * @param activitiVO
     * @return
     */
    private String createModelMetaInfo(ActivitiVO activitiVO) {
        JSONObject metaInfo = new JSONObject();
        metaInfo.put(ModelDataJsonConstants.MODEL_NAME, activitiVO.getModelName());
        metaInfo.put(ModelDataJsonConstants.MODEL_DESCRIPTION, activitiVO.getModelDescription());
        return metaInfo.toString();
    }

    /**
     * 添加模型时完善ModelEditorSource
     *
     * @param modelId
     */
    private void addModelEditorSource(String modelId) throws UnsupportedEncodingException {
        LOG.info("添加模型时完善ModelEditorSource入参模型ID：{}", modelId);
        JSONObject editorObject = new JSONObject();
        editorObject.put("id", "canvas");
        editorObject.put("resourceId", "canvas");

        JSONObject stencilSetObject = new JSONObject();
        stencilSetObject.put("namespace", "http://b3mn.org/stencilset/bpmn2.0#");

        editorObject.put("stencilset", stencilSetObject);
        repositoryService.addModelEditorSource(modelId, editorObject.toString().getBytes(ISystemConstant.CHARSET_UTF8));
    }

}
