package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.dao.oa.INodeFieldDao;
import ink.wgink.module.activiti.pojo.dtos.oa.NodeFieldDTO;
import ink.wgink.module.activiti.pojo.vos.oa.NodeFieldUpdateVO;
import ink.wgink.module.activiti.pojo.vos.oa.NodeFieldVO;
import ink.wgink.module.activiti.service.oa.INodeFieldService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Description: 表单字段与流程节点
 * @Author: WenG
 * @Date: 2022/3/17 20:09
 * @Version: 1.0
 **/
@Service
public class NodeFieldFieldServiceImpl extends DefaultBaseService implements INodeFieldService {

    @Autowired
    private INodeFieldDao nodeFieldDao;

    @Override
    public void save(NodeFieldUpdateVO nodeFieldUpdateVO) {
        LOG.debug("删除原有配置");
        delete(nodeFieldUpdateVO.getDeploymentId(), nodeFieldUpdateVO.getFormId(), nodeFieldUpdateVO.getFlowNodeId());
        LOG.debug("新增现有配置");
        save(nodeFieldUpdateVO.getDeploymentId(), nodeFieldUpdateVO.getFormId(), nodeFieldUpdateVO.getFlowNodeId(), nodeFieldUpdateVO.getFormFields());
    }

    @Override
    public void updateVisible(String nodeFieldId, Integer isVisible) {
        Map<String, Object> params = getHashMap(4);
        params.put("nodeFieldId", nodeFieldId);
        params.put("isVisible", isVisible);
        nodeFieldDao.update(params);
    }

    @Override
    public void updateEditable(String nodeFieldId, Integer isEditable) {
        Map<String, Object> params = getHashMap(4);
        params.put("nodeFieldId", nodeFieldId);
        params.put("isEditable", isEditable);
        nodeFieldDao.update(params);
    }

    @Override
    public List<NodeFieldDTO> listNodeByDeploymentId(String deploymentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("deploymentId", deploymentId);
        return nodeFieldDao.listNode(params);
    }

    @Override
    public List<NodeFieldDTO> listByDeploymentIdAndFlowNodeId(String deploymentId, String flowNodeId) {
        Map<String, Object> params = getHashMap(2);
        params.put("deploymentId", deploymentId);
        params.put("flowNodeId", flowNodeId);
        return nodeFieldDao.list(params);
    }

    /**
     * 保存
     *
     * @param deploymentId
     * @param formId
     * @param flowNodeId
     * @param formFields
     */
    private void save(String deploymentId, String formId, String flowNodeId, List<NodeFieldVO> formFields) {
        formFields.forEach(nodeFieldVO -> {
            Map<String, Object> params = HashMapUtil.beanToMap(nodeFieldVO);
            params.put("nodeFieldId", UUIDUtil.getUUID());
            params.put("deploymentId", deploymentId);
            params.put("formId", formId);
            params.put("flowNodeId", flowNodeId);
            nodeFieldDao.save(params);
        });
    }

    /**
     * 删除
     *
     * @param deploymentId 部署ID
     * @param formId       表单ID
     * @param flowNodeId   流程节点ID
     */
    private void delete(String deploymentId, String formId, String flowNodeId) {
        Map<String, Object> params = getHashMap(6);
        params.put("deploymentId", deploymentId);
        params.put("formId", formId);
        params.put("flowNodeId", flowNodeId);

        nodeFieldDao.delete(params);
    }

}
