package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.pojo.dtos.oa.OaTaskDTO;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.module.form.service.report.IFormReportService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.activiti.engine.task.TaskQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: OaFormReportServiceImpl
 * @Description: oa表单上报
 * @Author: wanggeng
 * @Date: 2022/3/22 22:05
 * @Version: 1.0
 */
@Service
public class OaFormReportServiceImpl extends DefaultBaseService implements IOaFormReportService {

    @Autowired
    private IFormReportService formReportService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private TaskService taskService;

    @Override
    public void save(String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        String uid = formReportService.saveAndReturnId(formCode, formVersion, params);
        LOG.debug("定义流程变量");
        Map<String, Object> variables = getHashMap(4);
        variables.put(KEY_FORM_CODE, formCode);
        variables.put(KEY_FORM_VERSION, formVersion);
        variables.put(KEY_REPORT_UID, uid);
        ProcessInstance processInstance = runtimeService.startProcessInstanceById(processDefinitionId, variables);
        // 更新表单数据的流程实例ID
        formReportService.updateProcessInstanceId(formCode, formVersion, uid, processInstance.getProcessInstanceId());
    }

    @Override
    public List<OaTaskDTO> listTaskOfMine() {
        String userId = securityComponent.getCurrentUser().getUserId();
        TaskQuery taskQuery = taskService.createTaskQuery().taskCandidateOrAssigned(userId);
        List<Task> tasks = taskQuery.list();
        List<OaTaskDTO> oaTaskDTOs = new ArrayList<>();
        tasks.forEach(task -> {
            OaTaskDTO oaTaskDTO = new OaTaskDTO();
            oaTaskDTO.setTaskId(task.getId());
            oaTaskDTO.setTaskName(task.getName());
            oaTaskDTO.setTaskDescription(task.getDescription());
            oaTaskDTO.setOwner(task.getOwner());

            Map<String, Object> variables = task.getProcessVariables();
            oaTaskDTO.setFormCode(variables.get(KEY_FORM_CODE).toString());
            oaTaskDTO.setFormVersion(Integer.parseInt(variables.get(KEY_FORM_VERSION).toString()));
            oaTaskDTO.setReportUid(variables.get(KEY_REPORT_UID).toString());

            oaTaskDTOs.add(oaTaskDTO);
        });
        return oaTaskDTOs;
    }


}
