package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.pojo.dtos.oa.OaProcdefDTO;
import ink.wgink.module.activiti.service.oa.IOaService;
import ink.wgink.module.form.pojo.dtos.design.FormDTO;
import ink.wgink.module.form.service.design.IFormService;
import ink.wgink.pojo.result.SuccessResultList;
import org.activiti.engine.FormService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: OaServiceImpl
 * @Description: oa管理
 * @Author: wanggeng
 * @Date: 2022/3/23 09:55
 * @Version: 1.0
 */
@Service
public class OaServiceImpl extends DefaultBaseService implements IOaService {

    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private FormService formService;
    @Autowired
    private IFormService iFormService;

    @Override
    public List<OaProcdefDTO> listProcdef() {
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.list();
        return listOaProcdef(processDefinitions);
    }

    @Override
    public SuccessResultList<List<OaProcdefDTO>> listPageProcdef(int page, int rows) {
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.listPage(page - 1, rows);
        List<OaProcdefDTO> oaProcdefDTOs = listOaProcdef(processDefinitions);
        return new SuccessResultList<>(oaProcdefDTOs, page, processDefinitionQuery.count());
    }

    /**
     * 流程定义列表
     *
     * @param processDefinitions
     * @return
     */
    private List<OaProcdefDTO> listOaProcdef(List<ProcessDefinition> processDefinitions) {
        if (processDefinitions.isEmpty()) {
            return new ArrayList<>();
        }
        List<String> formIds = new ArrayList<>();
        List<OaProcdefDTO> oaProcdefDTOs = new ArrayList<>();
        processDefinitions.forEach(processDefinition -> {
            String formKey = formService.getStartFormData(processDefinition.getId()).getFormKey();

            OaProcdefDTO oaProcdefDTO = new OaProcdefDTO();
            oaProcdefDTO.setId(processDefinition.getId());
            oaProcdefDTO.setDeploymentId(processDefinition.getDeploymentId());
            oaProcdefDTO.setName(processDefinition.getName());
            oaProcdefDTO.setFormKey(formKey);
            oaProcdefDTOs.add(oaProcdefDTO);
            if (StringUtils.isBlank(formKey)) {
                return;
            }
            formIds.add(formKey);
        });
        List<FormDTO> formDTOs = iFormService.listByIds(formIds);
        oaProcdefDTOs.forEach(oaProcdefDTO -> {
            formDTOs.forEach(formDTO -> {
                if (!StringUtils.equals(oaProcdefDTO.getFormKey(), formDTO.getFormId())) {
                    return;
                }
                oaProcdefDTO.setFormCode(formDTO.getFormCode());
                oaProcdefDTO.setFormVersion(formDTO.getFormVersion());
            });
        });
        return oaProcdefDTOs;
    }
}
