package ink.wgink.module.form.service.report.impl;

import freemarker.cache.StringTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.form.enums.design.FormStatusEnum;
import ink.wgink.module.form.pojo.pos.design.FormPO;
import ink.wgink.module.form.service.design.IFormService;
import ink.wgink.module.form.service.report.IFormReportRouteService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

/**
 * @ClassName: FormReportServiceImpl
 * @Description: 表单上报
 * @Author: wanggeng
 * @Date: 2022/3/10 7:33 PM
 * @Version: 1.0
 */
@Service
public class FormReportRouteServiceImpl extends DefaultBaseService implements IFormReportRouteService {

    @Autowired
    private FreeMarkerConfigurer freeMarkerConfigurer;
    @Autowired
    private IFormService formService;

    @Override
    public void save(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getSavePageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    @Override
    public void update(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getUpdatePageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    @Override
    public void show(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getShowPageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    @Override
    public void appSave(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getAppSavePageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    @Override
    public void appUpdate(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getAppUpdatePageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    @Override
    public void appShow(String formCode, Integer formVersion, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        FormPO formPO = getFormPO(formCode, formVersion);
        if (formPO == null) {
            return;
        }
        Map<String, Object> modal = getModal(formPO);
        showPage(modal, formCode, formPO.getAppShowPageCode(), httpSession, httpServletRequest, httpServletResponse);
    }

    /**
     * 展示页面
     *
     * @param modal
     * @param formCode
     * @param pageCode
     * @param httpSession
     * @param httpServletRequest
     * @param httpServletResponse
     */
    private void showPage(Map<String, Object> modal, String formCode, String pageCode, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        httpServletResponse.setContentType("text/html; charset=" + ISystemConstant.CHARSET_UTF8);
        try {
            Template template = getTemplate(formCode, pageCode);
            PrintWriter out = httpServletResponse.getWriter();
            template.process(modal, out);
        } catch (IOException e) {
            httpServletResponse.setStatus(HttpStatus.NOT_FOUND.value());
        } catch (TemplateException e) {
            LOG.error(e.getMessage(), e);
            throw new SystemException("模板错误");
        }
    }

    /**
     * 获取模型
     *
     * @return
     */
    private Map<String, Object> getModal(FormPO formPO) {
        Map<String, Object> modal = getHashMap(10);
        modal.put("contextPath", httpServletRequest.getContextPath() + "/");
        modal.put("formName", formPO.getFormName());
        modal.put("formCode", formPO.getFormCode());
        modal.put("formVersion", formPO.getFormVersion());
        return modal;
    }

    /**
     * 获取表单
     *
     * @param formCode
     * @param formVersion
     * @return
     */
    private FormPO getFormPO(String formCode, Integer formVersion) {
        FormPO formPO = formService.getPOByCodeAndVersion(formCode, formVersion);
        if (formPO == null) {
            httpServletResponse.setStatus(HttpStatus.NOT_FOUND.value());
            return null;
        }
        if (!StringUtils.equals(formPO.getFormStatus(), FormStatusEnum.ACTIVE.getValue())) {
            LOG.error("表单 {} 未激活", formCode);
            httpServletResponse.setStatus(HttpStatus.NOT_FOUND.value());
            return null;
        }
        return formPO;
    }

    /**
     * 获取配置
     *
     * @return
     */
    private Configuration getConfiguration() {
        StringTemplateLoader stringTemplateLoader = new StringTemplateLoader();
        Configuration configuration = freeMarkerConfigurer.getConfiguration();
        configuration.setTemplateLoader(stringTemplateLoader);
        configuration.setDefaultEncoding(ISystemConstant.CHARSET_UTF8);
        return configuration;
    }

    private Template getTemplate(String templateName, String templateCode) throws IOException {
        return new Template(templateName, templateCode, getConfiguration());
    }

}
