package ink.wgink.module.form.service.report.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.form.dao.formreport.IFormReportDao;
import ink.wgink.module.form.service.report.IFormReportService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: FormReportServiceImpl
 * @Description: 表单上报
 * @Author: wanggeng
 * @Date: 2022/3/14 5:30 PM
 * @Version: 1.0
 */
@Service
public class FormReportServiceImpl extends DefaultBaseService implements IFormReportService {

    @Autowired
    private IFormReportDao formReportDao;

    @Override
    public void save(String formCode, Integer formVersion, Map<String, Object> params) {
        saveAndReturnId(formCode, formVersion, params);
    }

    @Override
    public String saveAndReturnId(String formCode, Integer formVersion, Map<String, Object> params) {
        String uid = UUIDUtil.getUUID();
        List<String> inserts = new ArrayList<>();
        List<Object> values = new ArrayList<>();
        inserts.add("uid");
        values.add(uid);
        for (Map.Entry<String, Object> kv : params.entrySet()) {
            inserts.add(kv.getKey());
            values.add(kv.getValue());
        }
        String datetime = DateUtil.getTime();
        String userId = securityComponent.getCurrentUser().getUserId();
        inserts.add("gmt_create");
        values.add(datetime);
        inserts.add("creator");
        values.add(userId);
        inserts.add("gmt_modified");
        values.add(datetime);
        inserts.add("modifier");
        values.add(userId);
        inserts.add("is_delete");
        values.add(0);

        Map<String, Object> insertParams = getHashMap((params.size() + 5) * 2);
        insertParams.put("formCode", formCode);
        insertParams.put("formVersion", formVersion);
        insertParams.put("inserts", inserts);
        insertParams.put("values", values);
        formReportDao.save(insertParams);
        return uid;
    }

    @Override
    public void remove(String formCode, Integer formVersion, List<String> uids) {
        Map<String, Object> params = getHashMap(10);
        params.put("formCode", formCode);
        params.put("formVersion", formVersion);
        params.put("uids", uids);
        setUpdateInfo(params);

        formReportDao.remove(params);
    }

    @Override
    public void update(String formCode, Integer formVersion, String uid, Map<String, Object> params) {
        List<List<String>> kvList = new ArrayList<>();
        for (Map.Entry<String, Object> kv : params.entrySet()) {
            if (kv.getValue() == null) {
                continue;
            }
            kvList.add(Arrays.asList(new String[]{kv.getKey(), "" + kv.getValue()}));
        }
        kvList.add(Arrays.asList(new String[]{"gmt_modified", DateUtil.getTime()}));
        kvList.add(Arrays.asList(new String[]{"modifier", securityComponent.getCurrentUser().getUserId()}));

        Map<String, Object> updateParams = getHashMap((params.size() + 3) * 2);
        updateParams.put("uid", uid);
        updateParams.put("kvList", kvList);
        updateParams.put("formCode", formCode);
        updateParams.put("formVersion", formVersion);

        formReportDao.update(updateParams);
    }

    @Override
    public void updateProcessInstanceId(String formCode, Integer formVersion, String uid, String processInstanceId) {
        Map<String, Object> params = getHashMap(8);
        params.put("formCode", formCode);
        params.put("formVersion", formVersion);
        params.put("uid", uid);
        params.put("processInstanceId", processInstanceId);

        formReportDao.updateProcessInstanceId(params);
    }

    @Override
    public Map<String, Object> get(String formCode, Integer formVersion, String uid) {
        Map<String, Object> params = getHashMap(2);
        params.put("uid", uid);
        params.put("formCode", formCode);
        params.put("formVersion", formVersion);
        return formReportDao.get(params);
    }

    @Override
    public List<Map<String, Object>> list(String formCode, Integer formVersion, Map<String, Object> params) {
        List<String> queryList = conditionList(params);
        Map<String, Object> queryParams = getHashMap(6);
        queryParams.put("formCode", formCode);
        queryParams.put("formVersion", formVersion);
        queryParams.put("queryList", queryList);
        return formReportDao.list(queryParams);
    }

    @Override
    public SuccessResultList<List<Map<String, Object>>> listPage(String formCode, Integer formVersion, ListPage page) {
        List<String> queryList = conditionList(page.getParams());
        Map<String, Object> queryParams = getHashMap(6);
        queryParams.put("formCode", formCode);
        queryParams.put("formVersion", formVersion);
        queryParams.put("queryList", queryList);

        PageHelper.startPage(page.getPage(), page.getRows());
        List<Map<String, Object>> mapList = formReportDao.list(queryParams);
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<>(mapList);
        return new SuccessResultList<>(mapList, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    /**
     * 条件列表
     *
     * @param params
     * @return
     */
    private List<String> conditionList(Map<String, Object> params) {
        // 清除条件中的默认字段
        params.remove("requestUri");
        params.remove("requestHost");
        params.remove("requestPort");
        params.remove("keywords");
        params.remove("showLevel");
        params.remove("page");
        params.remove("rows");
        params.remove("is_delete");

        List<String> queryList = new ArrayList<>();
        for (Map.Entry<String, Object> kv : params.entrySet()) {
            String condition = getCondition(kv.getKey(), kv.getValue());
            if (StringUtils.isBlank(condition)) {
                continue;
            }
            queryList.add(condition);
        }
        return queryList;
    }

    private String getCondition(String key, Object value) {
        String conditionSQL;
        if (value == null) {
            return null;
        }
        String valueStr = "" + value;
        if (StringUtils.isBlank(valueStr)) {
            return null;
        }
        if (StringUtils.startsWith(valueStr, "*") && StringUtils.endsWith(valueStr, "*")) {
            // key LIKE '%keyword%'
            conditionSQL = key + " LIKE '%" + valueStr.substring(1, valueStr.length() - 1) + "%'";
        } else if (StringUtils.startsWith(valueStr, "*")) {
            // key LIKE '%keyword'
            conditionSQL = key + " LIKE '%" + valueStr.substring(1) + "'";
        } else if (StringUtils.endsWith(valueStr, "*")) {
            // key LIKE 'keyword%'
            conditionSQL = key + " LIKE '" + valueStr.substring(0, valueStr.length() - 1) + "%'";
        } else if (StringUtils.startsWith(valueStr, "lt;")) {
            // key < 'value'
            conditionSQL = key + " < '" + valueStr.substring(3) + "'";
        } else if (StringUtils.endsWith(valueStr, "gt;")) {
            // key > 'value'
            conditionSQL = key + " > '" + valueStr.substring(3) + "'";
        } else if (StringUtils.startsWith(valueStr, "le;")) {
            // key <= 'value'
            conditionSQL = key + " <= '" + valueStr.substring(3) + "'";
        } else if (StringUtils.endsWith(valueStr, "ge;")) {
            // key >= 'value'
            conditionSQL = key + " >= '" + valueStr.substring(3) + "'";
        } else if (StringUtils.endsWith(valueStr, "ne;")) {
            conditionSQL = key + " <> '" + valueStr.substring(3) + "'";
        } else {
            conditionSQL = key + " = '" + valueStr + "'";
        }
        return conditionSQL;
    }
}
